@@ -38,11 +38,17 @@ class Trainer:
             self.arguments.mode = self.determine_model_type(
                 self.arguments.model_name_or_path)
             if self.arguments.mode == "seq2seq":
-                print("Training seq2seq Language Model")
+                print("Training seq2seq language model")
             elif  self.arguments.mode == "clm":
-                print("Training causal Language Model")
+                print("Training causal language model")
             else:
                 print("Set TrainingArguments.mode to", self.arguments.mode)
+        try:
+            nproc = torch.distributed.get_world_size()
+            rank = torch.distributed.get_rank()
+            print("Process", rank, "of", nproc)
+        except Exception as e:
+            print("No distributed process detected:", e)
         # squeeze out more RAM to load larger models
         gc.collect()
         self.load_model()
@@ -113,6 +119,14 @@ class Trainer:
                     split=f"train[{self.arguments.validation_split}%:]",
                     cache_dir=self.arguments.cache_dir,
                 )
+        try:
+            nproc = torch.distributed.get_world_size()
+            if nproc > 1:
+                rank = torch.distributed.get_rank()
+                print("Distributed setting: rank", rank, "of", nproc)
+                self.raw_datasets = self.raw_datasets.shard(num_shards=nproc, index=rank)
+        except Exception as e:
+            print("No sharding of dataset:", e)
                 
     def GradCheckpoint(self):
         self.arguments.gradient_checkpointing = True
@@ -136,6 +150,9 @@ class Trainer:
             print(e, "(probably not a encoder-decoder model)")
         try:
             embedding_layers.append(self.model.transformer.wte)
+        except Exception as e:
+            print(e, "(probably not a GPT-Neo/GPT-J model)")
+        try:
             embedding_layers.append(self.model.transformer.wpe)
         except Exception as e:
             print(e, "(probably not a GPT-Neo model)")
@@ -286,6 +303,7 @@ class Trainer:
                 from_tf=bool(".ckpt" in self.arguments.model_name_or_path),
                 config=self.config,
                 cache_dir=self.arguments.cache_dir,
+                low_cpu_mem_usage=True
             )
         elif self.arguments.mode == "clm":
             self.model = transformers.AutoModelForCausalLM.from_pretrained(
@@ -293,6 +311,7 @@ class Trainer:
                 from_tf=bool(".ckpt" in self.arguments.model_name_or_path),
                 config=self.config,
                 cache_dir=self.arguments.cache_dir,
+                low_cpu_mem_usage=True
             )
         else:
             print("Model name:", self.arguments.model_name_or_path)
